local EventHandler = FlexBar2.OO.Mixin
{
	-- Custom event interface functions, act as advanced (and tweaked) versions of the AceEvent ones
	"RegisterEvent", -- Custom RegisterEvent function, Registers the event with AceEvent if it ain't already. It is table-based, read its code for details ;)
	"UnregisterEvent",
	"UnregisterAllEvents",

	"PreClick_RegisterCastingEvents", -- Register events needed to check/uncheck buttons after they have been clicked
	"ACTIONBAR_SLOT_CHANGED_UpdateTextures", -- Update button icons when an actionbar slot changed
	"SpellCast_End", -- Triggered whenever a spellcast ends, be it on the SPELLCAST_FAILED or SPELLCAST_STOP event
	"Channelling_Start", -- Triggered when the player starts to channel a spell
}


function EventHandler:Activate()
	self.Frame.Button:SetScript("PreClick", self.PreClick_RegisterCastingEvents);
	self:RegisterEvent("ACTIONBAR_SLOT_CHANGED", "ACTIONBAR_SLOT_CHANGED_UpdateTextures");
end


function EventHandler:RegisterEvent(Event, MethodName)
	if(not FlexBar2:IsEventRegistered(Event)) then
		FlexBar2[Event] = function(...)
			for n ,v in pairs(FlexBar2.EventTable[Event]) do
				for _,v in pairs(v) do
					n[v](n, ...);
				end
			end
		end
		FlexBar2:RegisterEvent(Event);
	end
	if(type(FlexBar2.EventTable[Event]) ~= "table") then
		FlexBar2.EventTable[Event] = {};
	end
	if(type(FlexBar2.EventTable[Event][self]) ~= "table") then
		FlexBar2.EventTable[Event][self] = {};
	end
	table.insert(FlexBar2.EventTable[Event][self], MethodName);
end

function EventHandler:UnregisterEvent(Event, MethodName)
	if(MethodName) then
		return FlexBar2:RemoveByValue(FlexBar2.EventTable[Event][self], MethodName);
	else
		FlexBar2.EventTable[Event][self] = {};
	end
end


function EventHandler:PreClick_RegisterCastingEvents() -- Can't use self directly here because this is a SetScript based trigger
	FlexBar2:Debug("PreClick_RegisterCastingEvents"); -- Register casting events to find out when casting ends
	local self = this:GetClass();
	self:RegisterEvent("UNIT_SPELLCAST_STOP", "SpellCast_End");
	self:RegisterEvent("UNIT_SPELLCAST_FAILED", "SpellCast_End");
	self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START", "Channelling_Start");
end


function EventHandler:ACTIONBAR_SLOT_CHANGED_UpdateTextures(Addon, Slot)
	-- Update actionbar slot texture if this button's slot corresponds with the changed slot(s)
	if(type(self.Settings) == "table" and type(self.Settings.Attributes) == "table" and self.Settings.Attributes.type == "action" and self.Settings.Attributes.action == Slot) then
		FlexBar2:Debug("ACTIONBAR_SLOT_CHANGED_UpdateTextures");
		FlexBar2:TriggerEvent("ACTIONBAR_SLOT_CHANGED_UpdateTextures");
		self:LoadCurrentTexture();
	end
end

function EventHandler:SpellCast_End(Addon, Unit)
	if(Unit == "player") then
		FlexBar2:Debug("SpellCast_End"); -- We will take care of unchecking the button and Unregistering casting events here
		FlexBar2:TriggerEvent("SpellCast_End");
		self.Frame.Button:SetChecked(false);
		self:UnregisterEvent("UNIT_SPELLCAST_STOP", "SpellCast_End");
		self:UnregisterEvent("UNIT_SPELLCAST_FAILED", "SpellCast_End");
	end
end

function EventHandler:Channelling_Stop(Addon, Unit)
	if(Unit == "player") then
		FlexBar2:Debug("Channelling_Stop"); -- We will take care of unchecking the button and Unregistering casting events here
		FlexBar2:TriggerEvent("Channelling_Stop");
		self.Frame.Button:SetChecked(true);
		-- Unregister the 2 casting stop event and use UNIT_SPELLCAST_CHANNEL_STOP instead
		self:UnregisterEvent("UNIT_SPELLCAST_STOP", "SpellCast_End");
		self:UnregisterEvent("UNIT_SPELLCAST_FAILED", "SpellCast_End");
	end
end

FlexBar2:RegisterClass(EventHandler);
