--[[
Name: FlexBar2 Core
Revision: $Rev$
Author: Cncfanatics
Description: Action Bars manager Mod, Core file
Dependencies: AceLibrary, AceOO-2.0, AceOO-2.0, AceEvent-2.0, AceAddon-2.0,AceDB-2.0, AceDebug-2.0, AceHook-2.1, AceConsole-2.0, AceLocale-2.2, Dewdrop-2.0, Tablet-2.0, FuBarPlugin-2.0
]]
FlexBar2 = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceDB-2.0", "AceHook-2.1", "AceDebug-2.0", "AceEvent-2.0"); -- Note: not registerring AceEvent-2.0 here, but in every class, as the event handling methods are in there anyway.
FlexBar2.OO = AceLibrary("AceOO-2.0");
FlexBar2.BabbleSpell = AceLibrary("Babble-Spell-2.2");
FlexBar2.PaintChips = AceLibrary("PaintChips-2.0");

FlexBar2:RegisterDB("FlexBar2_DB");
FlexBar2:RegisterDefaults("profile",
{
	Button = {},
	GenButtons = {}, -- List of classnames to generate by default
	ChannellingColor = {0,0,180}, -- Blue
	CastingColor = {180,180,0}, -- Yellow
	BuffedColor = {0,180,0}, -- Green
	DeBuffedColor = {180,0,0}, -- Red
});

FlexBar2.Activate = {}; -- This table will be a table of functions that will each get run once when a class is initiated

FlexBar2.Class =  {}; -- Table containing all classes that are registered and will thus be loaded

FlexBar2.EventTable = {};

FlexBar2.Buttons = {}; -- Contains the objects of all buttons

function FlexBar2:Startup()
	--[[
		TODO: Fill this text
	--]]
	FlexBar2.PaintChips:RegisterColor("Channelling", unpack(self.db.ChannellingColor));
	FlexBar2.PaintChips:RegisterColor("Casting", unpack(self.db.CastingColor));
	FlexBar2.PaintChips:RegisterColor("Buffed", unpack(self.db.BuffedColor));
	FlexBar2.PaintChips:RegisterColor("DeBuffed", unpack(self.db.DeBuffedColor));
	for i in pairs(self.db.profile.GenButtons) do
		self:Generate(i);
	end
end

function FlexBar2:OnInitialize()
	self:SetDebugging(true);
	self:Startup();
end

function FlexBar2:OnEnable()
end

function FlexBar2:OnDisable()
end

function FlexBar2:OnProfileEnable()
	self:Startup();
end

function FlexBar2:Generate(i, save)
	self:Debug("Generating " .. i);
	local Mixin = self.OO.Class(unpack(self.Class));
	function Mixin.prototype:init(...)
		Mixin.super.prototype.init(self);
		for _, v in ipairs(FlexBar2.Activate) do
			v(self, ...);
		end
	end
	self.Buttons[i] = Mixin:new(i);
	if(save) then
		self.db.profile.GenButtons[i] = true;
	end
end


function FlexBar2:RegisterClass(reference)
	--[[
		Register a class with FlexBar2, it will be used in any buttons that are generated after it has been registered (make sure you register your class early !)
		There is also a second, optional argument that adds a function that will be executed each time a class is initiated
	--]]
		table.insert(self.Class, reference);
		if(type(reference.Activate) == "function") then
			table.insert(self.Activate, reference.Activate);
		end
		return true;
end


-- Utility functions used on many parts of the addon
function FlexBar2:copyTable(t)
	local x = {}
	for k,v in pairs(t) do
		if(type(v) == "table") then
			x[k] = copyTable(v);
		else
			x[k] = v;
		end
		
	end
	setmetatable(x, getmetatable(t))
	return x
end

function FlexBar2:xor(a, b)
	return not (a ~= nil and a ~= false) == (b ~= nil and b ~= false);
end

function FlexBar2:RemoveByValue(tbl, val)
	for n, v in pairs(tbl) do
		if(v == val) then
			table.remove(tbl, n);
			return true;
		end
	end
	return false;
end
